package com.persistencia.hibernate;

import java.sql.Date;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

public class DemoQueriesHibernate {

	final static long DAY_IN_MILLIS = 1000 * 60 * 60 * 24;

	private static final SessionFactory sessionFactory;

	static {
		try {
			// Create the SessionFactory from hibernate.cfg.xml
			sessionFactory = new Configuration().configure()
					.buildSessionFactory();
		} catch (Throwable ex) {
			// Make sure you log the exception, as it might be swallowed
			System.err.println("Initial SessionFactory creation failed." + ex);
			throw new ExceptionInInitializerError(ex);
		}
	}

	public void insertEntidad(Entidad entidad) {
		Session session = sessionFactory.getCurrentSession();
		session.beginTransaction();
		session.save(entidad);
		session.getTransaction().commit();
	}

	public Integer selectMaxIdEntidad() {
		System.out.println("   - Obteniendo la sesion");
		Session session = sessionFactory.getCurrentSession();
		Integer id = new Integer(0);
		System.out
				.println("   - Obteniendo el max id (ejemplo de SQL estándar)");
		session.beginTransaction();
		SQLQuery q = session.createSQLQuery("select MAX(id) FROM DEMO_QUERIES");
		List resultado = q.list();
		if (resultado != null && resultado.size() > 0) {
			id = ((java.math.BigDecimal) resultado.get(0)).intValue() + 1;
		}
		return id;
	}

	public void listEntidad(java.util.Date fecha) {
		System.out.println("   - Obteniendo la sesion");
		Session session = sessionFactory.getCurrentSession();
		session.beginTransaction();
		Query q = session
				.createQuery("from e in class com.persistencia.hibernate.Entidad where e.fecha < :fecha");
		// Se crea una instancia como plantilla de los criterios de búsqueda
		Entidad plantilla = new Entidad();
		plantilla.setFecha(new java.sql.Date(fecha.getTime()));
		q.setProperties(plantilla);
		List entidades = q.list();
		for (Iterator iter = entidades.iterator(); iter.hasNext();) {
			Entidad entidad = (Entidad) iter.next();
			System.out.println("     o Id = " + entidad.getId()
					+ " - nombre = " + entidad.getNombre() + " - fecha = "
					+ entidad.getFecha() + "");
		}
	}

	/**
	 * Este método es idéntico a listEntidad pero se utiliza parámetros por
	 * nombre
	 * 
	 * @param fecha
	 */
	public void listEntidadPorNombre(java.util.Date fecha) {
		System.out.println("   - Obteniendo la sesion");
		Session session = sessionFactory.getCurrentSession();
		session.beginTransaction();
		Query q = session
				.createQuery("from e in class com.persistencia.hibernate.Entidad where e.fecha < :fecha");
		// se inyecta el valor del parámetro utilizando el nombre
		q.setParameter("fecha", new java.sql.Date(fecha.getTime()));
		List entidades = q.list();
		for (Iterator iter = entidades.iterator(); iter.hasNext();) {
			Entidad entidad = (Entidad) iter.next();
			System.out.println("     o Id = " + entidad.getId()
					+ " - nombre = " + entidad.getNombre() + " - fecha = "
					+ entidad.getFecha() + "");
		}
	}

	public static void main(String[] args) {
		System.out.println(" * Iniciando demo de queries JDBC\n");
		DemoQueriesHibernate demo = new DemoQueriesHibernate();

		System.out.println(" * Obtener el identificador mayor\n");
		Integer id = demo.selectMaxIdEntidad();

		System.out.println(" * Crear una nueva entidad\n");
		Entidad entidad = new Entidad();
		entidad.setId(id);
		entidad.setNombre("Ejemplo Hibernate\n");
		entidad.setFecha(new Date((new java.util.Date()).getTime()));

		System.out.println(" * Insertarla en la base de datos\n");
		demo.insertEntidad(entidad);

		System.out.println(" * Listar la base de datos por fecha\n");
		demo.listEntidad(new Date((new java.util.Date()).getTime()
				+ DAY_IN_MILLIS));

		System.out.println(" * Listar la base de datos por fecha\n");
		demo.listEntidadPorNombre(new Date((new java.util.Date()).getTime()
				+ DAY_IN_MILLIS));
	}

}
